diff --git a/drivers/dma/ti/k3-udma.c b/drivers/dma/ti/k3-udma.c
index cf653c2c8f37..bb19bf8009ae 100644
--- a/drivers/dma/ti/k3-udma.c
+++ b/drivers/dma/ti/k3-udma.c
@@ -2833,7 +2833,7 @@ static struct udma_desc *udma_alloc_tr_desc(struct udma_chan *uc,
 
 	tr_desc = hwdesc->cppi5_desc_vaddr;
 
-	if (uc->cyclic)
+	if (uc->cyclic && !uc->config.tr_trigger_type)
 		reload_count = CPPI5_INFO0_TRDESC_RLDCNT_INFINITE;
 
 	if (dir == DMA_DEV_TO_MEM && !uc->config.tr_trigger_type)
@@ -3559,6 +3559,70 @@ udma_prep_dma_cyclic_tr(struct udma_chan *uc, dma_addr_t buf_addr,
 	return d;
 }
 
+static struct udma_desc *
+udma_prep_dma_cyclic_triggered_tr(struct udma_chan *uc, dma_addr_t buf_addr,
+			size_t buf_len, size_t period_len,
+			enum dma_transfer_direction dir, unsigned long flags)
+{
+	struct udma_desc *d;
+	size_t tr_size;
+	dma_addr_t period_addr;
+	struct cppi5_tr_type15_t *tr_req;
+	unsigned int periods = buf_len / period_len;
+	u16 tr0_cnt0, tr0_cnt1, tr0_cnt2, tr0_cnt3, tr1_cnt0;
+	int num_tr;
+	int tr_idx = 0;
+
+	num_tr = udma_get_tr_counters(periods, 0, &tr0_cnt2, &tr0_cnt3,
+			&tr1_cnt0);
+	if (num_tr != 1 || !tr0_cnt2) {
+		dev_err(uc->ud->dev, "src size %zu and tgt size %zu are not supported\n",
+			period_len, buf_len);
+		return NULL;
+	}
+
+	tr0_cnt0 = uc->cfg.src_addr_width;
+	tr0_cnt1 = period_len / tr0_cnt0;
+
+	tr_size = sizeof(struct cppi5_tr_type15_t);
+	d = udma_alloc_tr_desc(uc, tr_size, periods * num_tr, dir);
+	if (!d)
+		return NULL;
+
+	tr_req = d->hwdesc[0].tr_req_base;
+	if (uc->ud->match_data->type == DMA_TYPE_UDMA)
+		period_addr = uc->cfg.src_addr;
+	else
+		period_addr = uc->cfg.src_addr |
+			((u64)uc->config.asel << K3_ADDRESS_ASEL_SHIFT);
+
+	cppi5_tr_init(&tr_req[tr_idx].flags, CPPI5_TR_TYPE15, false,
+		      true, CPPI5_TR_EVENT_SIZE_ICNT2_DEC, 0);
+	cppi5_tr_set_trigger(&tr_req[tr_idx].flags, uc->config.tr_trigger_type,
+			CPPI5_TR_TRIGGER_TYPE_ICNT2_DEC, 0, 0);
+
+	tr_req[tr_idx].addr = period_addr;
+	tr_req[tr_idx].icnt0 = tr0_cnt0;
+	tr_req[tr_idx].icnt1 = tr0_cnt1;
+	tr_req[tr_idx].icnt2 = periods;
+	tr_req[tr_idx].dim1 = 0;
+
+	tr_req[tr_idx].daddr = buf_addr |
+		((u64)uc->config.asel << K3_ADDRESS_ASEL_SHIFT);
+	tr_req[tr_idx].dicnt0 = tr0_cnt0;
+	tr_req[tr_idx].dicnt1 = tr0_cnt1;
+	tr_req[tr_idx].dicnt2 = periods;
+	tr_req[tr_idx].ddim1 = tr0_cnt0;
+	tr_req[tr_idx].ddim2 = period_len;
+
+	if (!(flags & DMA_PREP_INTERRUPT))
+		cppi5_tr_csf_set(&tr_req[tr_idx].flags, CPPI5_TR_CSF_SUPR_EVT);
+
+	cppi5_tr_csf_set(&tr_req[tr_idx].flags, CPPI5_TR_CSF_EOP);
+
+	return d;
+}
+
 static struct udma_desc *
 udma_prep_dma_cyclic_pkt(struct udma_chan *uc, dma_addr_t buf_addr,
 			 size_t buf_len, size_t period_len,
@@ -3637,6 +3701,7 @@ udma_prep_dma_cyclic(struct dma_chan *chan, dma_addr_t buf_addr, size_t buf_len,
 	struct udma_desc *d;
 	u32 burst;
 
+#if 0
 	if (dir != uc->config.dir) {
 		dev_err(chan->device->dev,
 			"%s: chan%d is for %s, not supporting %s\n",
@@ -3645,6 +3710,7 @@ udma_prep_dma_cyclic(struct dma_chan *chan, dma_addr_t buf_addr, size_t buf_len,
 			dmaengine_get_direction_text(dir));
 		return NULL;
 	}
+#endif
 
 	uc->cyclic = true;
 
@@ -3665,14 +3731,20 @@ udma_prep_dma_cyclic(struct dma_chan *chan, dma_addr_t buf_addr, size_t buf_len,
 	if (uc->config.pkt_mode)
 		d = udma_prep_dma_cyclic_pkt(uc, buf_addr, buf_len, period_len,
 					     dir, flags);
-	else
+	else if (!uc->config.tr_trigger_type)
 		d = udma_prep_dma_cyclic_tr(uc, buf_addr, buf_len, period_len,
 					    dir, flags);
+	else
+		d = udma_prep_dma_cyclic_triggered_tr(uc, buf_addr, buf_len,
+					              period_len, dir, flags);
 
 	if (!d)
 		return NULL;
 
-	d->sglen = buf_len / period_len;
+	if (!uc->config.tr_trigger_type)
+		d->sglen = buf_len / period_len;
+	else
+		d->sglen = 1;
 
 	d->dir = dir;
 	d->residue = buf_len;
-- 
2.34.1

